package Test::Class::Moose::Tutorial
$Test::Class::Moose::Tutorial::VERSION = '0.52';
# ABSTRACT: A starting guide for Test::Class::Moose

# there is no code here, but we're moving this from .pod to .pm to try to work
# around a strange bug where this is showing up instead of main docs on
# metacpan and cpan

1;

__END__

=pod

=cut
